@@ -1,6 +1,5 @@
 #include "EditTableDialog.h"
 #include "ui_EditTableDialog.h"
-#include "SQLiteSyntaxHighlighter.h"
 #include "sqlitetablemodel.h"
 #include "sqlitedb.h"
 
@@ -24,8 +23,6 @@ EditTableDialog::EditTableDialog(DBBrowserDB* db, const QString& tableName, bool
     ui->widgetExtension->setVisible(false);
     connect(ui->treeWidget, SIGNAL(itemChanged(QTreeWidgetItem*,int)),this,SLOT(itemChanged(QTreeWidgetItem*,int)));
 
-    m_sqliteSyntaxHighlighter = new SQLiteSyntaxHighlighter(ui->sqlTextEdit->document());
-
     // Editing an existing table?
     if(m_bNewTable == false)
     {
@@ -108,7 +105,15 @@ void EditTableDialog::populateFields()
         tbitem->setCheckState(kPrimaryKey, f->primaryKey() ? Qt::Checked : Qt::Unchecked);
         tbitem->setCheckState(kAutoIncrement, f->autoIncrement() ? Qt::Checked : Qt::Unchecked);
         tbitem->setCheckState(kUnique, f->unique() ? Qt::Checked : Qt::Unchecked);
-        tbitem->setText(kDefault, f->defaultValue());
+
+        // For the default value check if it is surrounded by parentheses and if that's the case
+        // add a '=' character before the entire string to match the input format we're expecting
+        // from the user when using functions in the default value field.
+        if(f->defaultValue().startsWith('(') && f->defaultValue().endsWith(')'))
+            tbitem->setText(kDefault, "=" + f->defaultValue());
+        else
+            tbitem->setText(kDefault, f->defaultValue());
+
         tbitem->setText(kCheck, f->check());
         tbitem->setText(kForeignKey, f->foreignKey());
         ui->treeWidget->addTopLevelItem(tbitem);
@@ -161,8 +166,7 @@ void EditTableDialog::reject()
 
 void EditTableDialog::updateSqlText()
 {
-    ui->sqlTextEdit->clear();
-    ui->sqlTextEdit->insertPlainText(m_table.sql());
+    ui->sqlTextEdit->setText(m_table.sql());
 }
 
 void EditTableDialog::checkInput()
@@ -544,15 +548,14 @@ void EditTableDialog::setWithoutRowid(bool without_rowid)
 {
     if(without_rowid)
     {
-        // Before setting the without rowid flag, first perform a check to see if the table meets all thr required criteria for without rowid tables
+        // Before setting the without rowid flag, first perform a check to see if the table meets all the required criteria for without rowid tables
         int pk = m_table.findPk();
-        if(pk == -1 || m_table.fields().at(pk)->isInteger() == false || m_table.fields().at(pk)->autoIncrement())
+        if(pk == -1 || m_table.fields().at(pk)->autoIncrement())
         {
             QMessageBox::information(this, QApplication::applicationName(),
                                      tr("Please add a field which meets the following criteria before setting the without rowid flag:\n"
                                         " - Primary key flag set\n"
-                                        " - Auto incremenct disabled\n"
-                                        " - Type INTEGER"));
+                                        " - Auto increment disabled"));
             ui->checkWithoutRowid->setChecked(false);
             return;
         }
