@@ -3,6 +3,7 @@
 
 #include <QTableView>
 #include "FilterTableHeader.h"
+#include <QSet>
 
 class ExtendedTableWidget : public QTableView
 {
@@ -13,6 +14,9 @@ public:
 
     FilterTableHeader* filterHeader() { return m_tableHeader; }
 
+public:
+    QSet<int> selectedCols();
+
 private:
     void copy();
     int numVisibleRows();
