@@ -8,17 +8,17 @@
 #include "ExportCsvDialog.h"
 #include "PreferencesDialog.h"
 #include "EditDialog.h"
-#include "SQLiteSyntaxHighlighter.h"
-#include "sqltextedit.h"
 #include "sqlitetablemodel.h"
 #include "SqlExecutionArea.h"
 #include "VacuumDialog.h"
 #include "DbStructureModel.h"
-#include "gen_version.h"
+#include "version.h"
 #include "sqlite.h"
 #include "CipherDialog.h"
+#include "ExportSqlDialog.h"
+#include "SqlUiLexer.h"
+#include "FileDialog.h"
 
-#include <QFileDialog>
 #include <QFile>
 #include <QApplication>
 #include <QTextStream>
@@ -41,7 +41,8 @@
 #include <QXmlStreamWriter>
 #include <QInputDialog>
 #include <QProgressDialog>
-
+#include <QTextEdit>
+#include <QClipboard>
 
 MainWindow::MainWindow(QWidget* parent)
     : QMainWindow(parent),
@@ -103,9 +104,11 @@ void MainWindow::init()
 
     // Create popup menus
     popupTableMenu = new QMenu(this);
+    popupTableMenu->addAction(ui->actionEditBrowseTable);
     popupTableMenu->addAction(ui->editModifyTableAction);
     popupTableMenu->addAction(ui->editDeleteObjectAction);
     popupTableMenu->addSeparator();
+    popupTableMenu->addAction(ui->actionEditCopyCreateStatement);
     popupTableMenu->addAction(ui->actionExportCsvPopup);
 
     popupSaveSqlFileMenu = new QMenu(this);
@@ -160,6 +163,8 @@ void MainWindow::init()
     restoreState(PreferencesDialog::getSettingsValue("MainWindow", "windowState").toByteArray());
     ui->comboLogSubmittedBy->setCurrentIndex(ui->comboLogSubmittedBy->findText(PreferencesDialog::getSettingsValue("SQLLogDock", "Log").toString()));
     ui->splitterForPlot->restoreState(PreferencesDialog::getSettingsValue("PlotDock", "splitterSize").toByteArray());
+    ui->comboLineType->setCurrentIndex(PreferencesDialog::getSettingsValue("PlotDock", "lineType").toInt());
+    ui->comboPointShape->setCurrentIndex(PreferencesDialog::getSettingsValue("PlotDock", "pointShape").toInt());
 
     // plot widgets
     ui->treePlotColumns->setSelectionMode(QAbstractItemView::NoSelection);
@@ -172,31 +177,24 @@ void MainWindow::init()
     reloadSettings();
 
 #ifdef CHECKNEWVERSION
-    // Check for a new release version, usually only enabled on windows
-    m_NetworkManager = new QNetworkAccessManager(this);
-    QObject::connect(m_NetworkManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(httpresponse(QNetworkReply*)));
+    // Check for a new version if automatic update check aren't disabled in the settings dialog
+    if(PreferencesDialog::getSettingsValue("checkversion", "enabled").toBool())
+    {
+        // Check for a new release version, usually only enabled on windows
+        m_NetworkManager = new QNetworkAccessManager(this);
+        QObject::connect(m_NetworkManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(httpresponse(QNetworkReply*)));
 
-    QUrl url("https://raw.github.com/sqlitebrowser/sqlitebrowser/master/currentrelease");
-    m_NetworkManager->get(QNetworkRequest(url));
+        QUrl url("https://raw.githubusercontent.com/sqlitebrowser/sqlitebrowser/master/currentrelease");
+        m_NetworkManager->get(QNetworkRequest(url));
+    }
 #endif
 
 #ifndef ENABLE_SQLCIPHER
-    // Only show encrpytion menu action when SQLCipher support is enabled
+    // Only show encryption menu action when SQLCipher support is enabled
     ui->actionEncryption->setVisible(false);
 #endif
 }
 
-void MainWindow::clearCompleterModelsFields()
-{
-    for(SqlTextEdit::FieldCompleterModelMap::iterator it = completerModelsFields.begin();
-        it != completerModelsFields.end();
-        ++it)
-    {
-        delete *it;
-    }
-    completerModelsFields.clear();
-}
-
 bool MainWindow::fileOpen(const QString& fileName, bool dontAddToRecentFiles)
 {
     bool retval = false;
@@ -204,10 +202,13 @@ bool MainWindow::fileOpen(const QString& fileName, bool dontAddToRecentFiles)
     QString wFile = fileName;
     if (!QFile::exists(wFile))
     {
-        wFile = QFileDialog::getOpenFileName(
+        wFile = FileDialog::getOpenFileName(
                     this,
-                    tr("Choose a database file"),
-                    PreferencesDialog::getSettingsValue("db", "defaultlocation").toString());
+                    tr("Choose a database file")
+#ifndef Q_OS_MAC // Filters on OS X are buggy
+                    , tr("SQLite database files (*.db *.sqlite *.sqlite3 *.db3);;All files (*)")
+#endif
+                    );
     }
     if(QFile::exists(wFile) )
     {
@@ -245,7 +246,7 @@ bool MainWindow::fileOpen(const QString& fileName, bool dontAddToRecentFiles)
 
 void MainWindow::fileNew()
 {
-    QString fileName = QFileDialog::getSaveFileName(this, tr("Choose a filename to save under"), PreferencesDialog::getSettingsValue("db", "defaultlocation").toString());
+    QString fileName = FileDialog::getSaveFileName(this, tr("Choose a filename to save under"));
     if(!fileName.isEmpty())
     {
         if(QFile::exists(fileName))
@@ -264,9 +265,6 @@ void MainWindow::fileNew()
 
 void MainWindow::populateStructure()
 {
-    completerModelTables.clear();
-    clearCompleterModelsFields();
-
     // Refresh the structure tab
     db.updateSchema();
     dbStructureModel->reloadData(&db);
@@ -276,49 +274,28 @@ void MainWindow::populateStructure()
     if(!db.isOpen())
         return;
 
-    QStringList tblnames = db.getBrowsableObjectNames();
-    ui->editLogUser->syntaxHighlighter()->setTableNames(tblnames);
-    ui->editLogApplication->syntaxHighlighter()->setTableNames(tblnames);
-
-    // setup models for sqltextedit autocomplete
-    completerModelTables.setRowCount(tblnames.count());
-    completerModelTables.setColumnCount(1);
-
+    // Update table and column names for syntax highlighting
     objectMap tab = db.getBrowsableObjects();
-    int row = 0;
-    for(objectMap::ConstIterator it=tab.begin(); it!=tab.end(); ++it, ++row)
+    SqlUiLexer::TablesAndColumnsMap tablesToColumnsMap;
+    for(objectMap::ConstIterator it=tab.begin(); it!=tab.end(); ++it)
     {
-        QString sName = it.value().getname();
-        QStandardItem* item = new QStandardItem(sName);
-        item->setIcon(QIcon(QString(":icons/%1").arg(it.value().gettype())));
-        completerModelTables.setItem(row, 0, item);
-
-        // If it is a table add the field Nodes
+        // If it is a table or a view add the fields
         if((*it).gettype() == "table" || (*it).gettype() == "view")
         {
-            QStandardItemModel* tablefieldmodel = new QStandardItemModel();
-            tablefieldmodel->setRowCount((*it).table.fields().count());
-            tablefieldmodel->setColumnCount(1);
+            QString objectname = it.value().getname();
 
-            int fldrow = 0;
-            for(int i=0; i < (*it).table.fields().size(); ++i, ++fldrow)
+            for(int i=0; i < (*it).table.fields().size(); ++i)
             {
                 QString fieldname = (*it).table.fields().at(i)->name();
-                QStandardItem* fldItem = new QStandardItem(fieldname);
-                fldItem->setIcon(QIcon(":/icons/field"));
-                tablefieldmodel->setItem(fldrow, 0, fldItem);
+                tablesToColumnsMap[objectname].append(fieldname);
             }
-            completerModelsFields.insert(sName.toLower(), tablefieldmodel);
         }
-
-    }
-    for(int i=0; i < ui->tabSqlAreas->count(); ++i)
-    {
-        SqlExecutionArea* sqlarea = qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->widget(i));
-        sqlarea->setTableNames(tblnames);
-        sqlarea->getEditor()->setDefaultCompleterModel(&completerModelTables);
-        sqlarea->getEditor()->insertFieldCompleterModels(completerModelsFields);
     }
+    SqlTextEdit::sqlLexer->setTableNames(tablesToColumnsMap);
+    ui->editLogApplication->reloadKeywords();
+    ui->editLogUser->reloadKeywords();
+    for(int i=0;i<ui->tabSqlAreas->count();i++)
+        qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->widget(i))->getEditor()->reloadKeywords();
 
     // Resize SQL column to fit contents
     ui->dbTreeWidget->resizeColumnToContents(3);
@@ -415,7 +392,7 @@ void MainWindow::resetBrowser()
     curBrowseOrderByIndex = 0;
     curBrowseOrderByMode = Qt::AscendingOrder;
     m_browseTableModel->sort(curBrowseOrderByIndex, curBrowseOrderByMode);
-    populateTable(ui->comboBrowseTable->currentText());
+    populateTable(ui->comboBrowseTable->currentText(), true);
 }
 
 void MainWindow::fileClose()
@@ -457,7 +434,8 @@ void MainWindow::closeEvent( QCloseEvent* event )
         PreferencesDialog::setSettingsValue("MainWindow", "windowState", saveState());
         PreferencesDialog::setSettingsValue("SQLLogDock", "Log", ui->comboLogSubmittedBy->currentText());
         PreferencesDialog::setSettingsValue("PlotDock", "splitterSize", ui->splitterForPlot->saveState());
-        clearCompleterModelsFields();
+        PreferencesDialog::setSettingsValue("PlotDock", "lineType", ui->comboLineType->currentIndex());
+        PreferencesDialog::setSettingsValue("PlotDock", "pointShape", ui->comboPointShape->currentIndex());
         QMainWindow::closeEvent(event);
     } else {
         event->ignore();
@@ -715,15 +693,24 @@ void MainWindow::executeQuery()
     // Get SQL code to execute. This depends on the button that's been pressed
     QString query;
     bool singleStep = false;
+    int execution_start_line = 0;
+    int execution_start_index = 0;
     if(sender()->objectName() == "actionSqlExecuteLine")
     {
-        query = sqlWidget->getEditor()->document()->toPlainText().mid(sqlWidget->getEditor()->textCursor().block().position());
+        int cursor_line, cursor_index;
+        sqlWidget->getEditor()->getCursorPosition(&cursor_line, &cursor_index);
+        execution_start_line = cursor_line;
+        while(cursor_line < sqlWidget->getEditor()->lines())
+            query += sqlWidget->getEditor()->text(cursor_line++);
         singleStep = true;
     } else {
         // if a part of the query is selected, we will only execute this part
         query = sqlWidget->getSelectedSql();
+        int dummy;
         if(query.isEmpty())
             query = sqlWidget->getSql();
+        else
+            sqlWidget->getEditor()->getSelection(&execution_start_line, &execution_start_index, &dummy, &dummy);
     }
     if (query.isEmpty())
         return;
@@ -745,17 +732,25 @@ void MainWindow::executeQuery()
     // gets finalized, see http://www.sqlite.org/lang_transaction.html
     db.setRestorePoint();
 
+    // Remove any error indicators
+    sqlWidget->getEditor()->clearIndicatorRange(0, 0,
+                                                sqlWidget->getEditor()->lines(), sqlWidget->getEditor()->lineLength(sqlWidget->getEditor()->lines()),
+                                                sqlWidget->getEditor()->getErrorIndicatorNumber());
+
     //Accept multi-line queries, by looping until the tail is empty
     QElapsedTimer timer;
     timer.start();
     do
     {
+        int tail_length_before = tail_length;
         const char* qbegin = tail;
-        sql3status = sqlite3_prepare_v2(db._db,tail, tail_length,
-                            &vm, &tail);
+        sql3status = sqlite3_prepare_v2(db._db,tail, tail_length, &vm, &tail);
         QString queryPart = QString::fromUtf8(qbegin, tail - qbegin);
         tail_length -= (tail - qbegin);
-        if (sql3status == SQLITE_OK){
+        int execution_end_index = execution_start_index + tail_length_before - tail_length;
+
+        if (sql3status == SQLITE_OK)
+        {
             sql3status = sqlite3_step(vm);
             sqlite3_finalize(vm);
 
@@ -773,8 +768,9 @@ void MainWindow::executeQuery()
                 sqlWidget->getModel()->setQuery(queryPart);
                 if(sqlWidget->getModel()->valid())
                 {
-                    statusMessage = tr("%1 Rows returned from: %2 (took %3ms)").arg(
-                                sqlWidget->getModel()->totalRowCount()).arg(queryPart.trimmed()).arg(timer.elapsed());
+                    // The query takes the last placeholder as it may itself contain the sequence '%' + number
+                    statusMessage = tr("%1 Rows returned from: %3 (took %2ms)").arg(
+                                sqlWidget->getModel()->totalRowCount()).arg(timer.elapsed()).arg(queryPart.trimmed());
                     sqlWidget->enableSaveButton(true);
                     sql3status = SQLITE_OK;
                 }
@@ -807,7 +803,11 @@ void MainWindow::executeQuery()
         } else {
             statusMessage = QString::fromUtf8((const char*)sqlite3_errmsg(db._db)) +
                     ": " + queryPart;
+            sqlWidget->getEditor()->fillIndicatorRange(execution_start_line, execution_start_index, execution_start_line, execution_end_index,
+                                                       sqlWidget->getEditor()->getErrorIndicatorNumber());
         }
+
+        execution_start_index = execution_end_index;
     } while( tail && *tail != 0 && (sql3status == SQLITE_OK || sql3status == SQLITE_DONE));
     sqlWidget->finishExecution(statusMessage);
     updatePlot(sqlWidget->getModel());
@@ -831,10 +831,9 @@ void MainWindow::mainTabSelected(int tabindex)
 
 void MainWindow::importTableFromCSV()
 {
-    QString wFile = QFileDialog::getOpenFileName(
+    QString wFile = FileDialog::getOpenFileName(
                 this,
                 tr("Choose a text file"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                 tr("Text files(*.csv *.txt);;All files(*)"));
 
     if (QFile::exists(wFile) )
@@ -892,28 +891,20 @@ void MainWindow::fileRevert()
 
 void MainWindow::exportDatabaseToSQL()
 {
-    QString fileName = QFileDialog::getSaveFileName(
-                this,
-                tr("Choose a filename to export"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
-                tr("Text files(*.sql *.txt)"));
+    QString current_table;
+    if(ui->mainTab->currentIndex() == 1)
+        current_table = ui->comboBrowseTable->currentText();
 
-    if(fileName.size())
-    {
-        if(!db.dump(fileName))
-            QMessageBox::warning(this, QApplication::applicationName(), tr("Export cancelled or failed."));
-        else
-            QMessageBox::information(this, QApplication::applicationName(), tr("Export completed."));
-    }
+    ExportSqlDialog dialog(&db, this, current_table);
+    dialog.exec();
 }
 
 void MainWindow::importDatabaseFromSQL()
 {
     // Get file name to import
-    QString fileName = QFileDialog::getOpenFileName(
+    QString fileName = FileDialog::getOpenFileName(
                 this,
                 tr("Choose a file to import"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                 tr("Text files(*.sql *.txt);;All files(*)"));
 
     // Cancel when file doesn't exist
@@ -928,10 +919,9 @@ void MainWindow::importDatabaseFromSQL()
                                                "If you answer no we will attempt to import the data in the SQL file to the current database."),
                                             QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) || !db.isOpen())
     {
-        newDbFile = QFileDialog::getSaveFileName(
+        newDbFile = FileDialog::getSaveFileName(
                     this,
-                    tr("Choose a filename to save under"),
-                    PreferencesDialog::getSettingsValue("db", "defaultlocation").toString());
+                    tr("Choose a filename to save under"));
         if(QFile::exists(newDbFile))
         {
             QMessageBox::information(this, QApplication::applicationName(), tr("File %1 already exists. Please choose a different name.").arg(newDbFile));
@@ -993,6 +983,7 @@ void MainWindow::changeTreeSelection()
     // Just assume first that something's selected that can not be edited at all
     ui->editDeleteObjectAction->setEnabled(false);
     ui->editModifyTableAction->setEnabled(false);
+    ui->actionEditBrowseTable->setEnabled(false);
 
     if(!ui->dbTreeWidget->currentIndex().isValid())
         return;
@@ -1017,7 +1008,11 @@ void MainWindow::changeTreeSelection()
     } else if(type == "view" || type == "trigger" || type == "index") {
         ui->editDeleteObjectAction->setEnabled(true);
     }
-    ui->actionExportCsvPopup->setEnabled(type == "table" || type == "view");
+    if(type == "table" || type == "view")
+    {
+        ui->actionEditBrowseTable->setEnabled(true);
+        ui->actionExportCsvPopup->setEnabled(true);
+    }
 }
 
 void MainWindow::openRecentFile()
@@ -1112,9 +1107,12 @@ void MainWindow::activateFields(bool enable)
     ui->actionSqlExecuteLine->setEnabled(enable);
     ui->actionSqlOpenFile->setEnabled(enable);
     ui->actionSqlOpenTab->setEnabled(enable);
+    ui->actionSqlSaveFilePopup->setEnabled(enable);
+    ui->actionSqlSaveFileAs->setEnabled(enable);
     ui->actionSqlSaveFile->setEnabled(enable);
     ui->actionSaveProject->setEnabled(enable);
     ui->actionEncryption->setEnabled(enable);
+    ui->buttonClearFilters->setEnabled(enable);
 }
 
 void MainWindow::browseTableHeaderClicked(int logicalindex)
@@ -1126,7 +1124,7 @@ void MainWindow::browseTableHeaderClicked(int logicalindex)
     // instead of the column name we just use the column index, +2 because 'rowid, *' is the projection
     curBrowseOrderByIndex = logicalindex;
     curBrowseOrderByMode = curBrowseOrderByMode == Qt::AscendingOrder ? Qt::DescendingOrder : Qt::AscendingOrder;
-    m_browseTableModel->sort(curBrowseOrderByIndex, curBrowseOrderByMode);
+    ui->dataTable->sortByColumn(curBrowseOrderByIndex, curBrowseOrderByMode);
 
     // select the first item in the column so the header is bold
     // we might try to select the last selected item
@@ -1217,10 +1215,10 @@ void MainWindow::logSql(const QString& sql, int msgtype)
 {
     if(msgtype == kLogMsg_User)
     {
-        ui->editLogUser->appendPlainText(sql);
+        ui->editLogUser->append(sql + "\n");
         ui->editLogUser->verticalScrollBar()->setValue(ui->editLogUser->verticalScrollBar()->maximum());
     } else {
-        ui->editLogApplication->appendPlainText(sql);
+        ui->editLogApplication->append(sql + "\n");
         ui->editLogApplication->verticalScrollBar()->setValue(ui->editLogApplication->verticalScrollBar()->maximum());
     }
 }
@@ -1246,9 +1244,6 @@ unsigned int MainWindow::openSqlTab(bool resetCounter)
 
     // Create new tab, add it to the tab widget and select it
     SqlExecutionArea* w = new SqlExecutionArea(this, &db);
-    w->setTableNames(db.getBrowsableObjectNames());
-    w->getEditor()->setDefaultCompleterModel(&completerModelTables);
-    w->getEditor()->insertFieldCompleterModels(completerModelsFields);
     int index = ui->tabSqlAreas->addTab(w, QString("SQL %1").arg(++tabNumber));
     ui->tabSqlAreas->setCurrentIndex(index);
     w->getEditor()->setFocus();
@@ -1258,10 +1253,9 @@ unsigned int MainWindow::openSqlTab(bool resetCounter)
 
 void MainWindow::openSqlFile()
 {
-    QString file = QFileDialog::getOpenFileName(
+    QString file = FileDialog::getOpenFileName(
                 this,
                 tr("Select SQL file to open"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                 tr("Text files(*.sql *.txt);;All files(*)"));
 
     if(QFile::exists(file))
@@ -1269,7 +1263,7 @@ void MainWindow::openSqlFile()
         QFile f(file);
         f.open(QIODevice::ReadOnly);
 
-        // Decide wether to open a new tab or take the current one
+        // Decide whether to open a new tab or take the current one
         unsigned int index;
         SqlExecutionArea* current_tab = qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->currentWidget());
         if(current_tab->getSql().isEmpty() && current_tab->getModel()->rowCount() == 0)
@@ -1278,7 +1272,7 @@ void MainWindow::openSqlFile()
             index = openSqlTab();
 
         SqlExecutionArea* sqlarea = qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->widget(index));
-        sqlarea->getEditor()->setPlainText(f.readAll());
+        sqlarea->getEditor()->setText(f.readAll());
         sqlarea->setFileName(file);
         QFileInfo fileinfo(file);
         ui->tabSqlAreas->setTabText(index, fileinfo.fileName());
@@ -1305,10 +1299,9 @@ void MainWindow::saveSqlFile()
 
 void MainWindow::saveSqlFileAs()
 {
-    QString file = QFileDialog::getSaveFileName(
+    QString file = FileDialog::getSaveFileName(
                 this,
                 tr("Select file name"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                 tr("Text files(*.sql *.txt);;All files(*)"));
 
     if(!file.isEmpty())
@@ -1321,10 +1314,9 @@ void MainWindow::saveSqlFileAs()
 
 void MainWindow::loadExtension()
 {
-    QString file = QFileDialog::getOpenFileName(
+    QString file = FileDialog::getOpenFileName(
                 this,
                 tr("Select extension file"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                 tr("Extensions(*.so *.dll);;All files(*)"));
 
     if(file.isEmpty())
@@ -1353,7 +1345,6 @@ void MainWindow::reloadSettings()
 {
     // Read settings
     int prefetch_size = PreferencesDialog::getSettingsValue("db", "prefetchsize").toInt();
-    int edit_fontsize = PreferencesDialog::getSettingsValue("editor", "fontsize").toInt();
     int log_fontsize = PreferencesDialog::getSettingsValue("log", "fontsize").toInt();
 
     QFont logfont("Monospace");
@@ -1367,13 +1358,12 @@ void MainWindow::reloadSettings()
         SqlExecutionArea* sqlArea = qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->widget(i));
         sqlArea->getModel()->setChunkSize(prefetch_size);
         sqlArea->getResultView()->setFont(logfont);
-
-        QFont font = sqlArea->getEditor()->font();
-        font.setPointSize(edit_fontsize);
-        sqlArea->getEditor()->setFont(font);
+        sqlArea->getEditor()->reloadSettings();
     }
 
     // Set font for SQL logs
+    ui->editLogApplication->reloadSettings();
+    ui->editLogUser->reloadSettings();
     ui->editLogApplication->setFont(logfont);
     ui->editLogUser->setFont(logfont);
 
@@ -1387,8 +1377,6 @@ void MainWindow::reloadSettings()
 
 void MainWindow::httpresponse(QNetworkReply *reply)
 {
-    //QVariant statuscode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
-
     if(reply->error() == QNetworkReply::NoError)
     {
         // Check for redirect
@@ -1431,13 +1419,12 @@ void MainWindow::httpresponse(QNetworkReply *reply)
         if(newversion)
         {
             QSettings settings(QApplication::organizationName(), QApplication::organizationName());
-            bool disablecheck = settings.value("checkversion/disable", false).toBool();
-            int ignmajor = settings.value("checkversion/major", 999).toInt();
-            int ignminor = settings.value("checkversion/minor", 0).toInt();
-            int ignpatch = settings.value("checkversion/patch", 0).toInt();
+            int ignmajor = settings.value("checkversion/ignmajor", 999).toInt();
+            int ignminor = settings.value("checkversion/ignminor", 0).toInt();
+            int ignpatch = settings.value("checkversion/ignpatch", 0).toInt();
 
             // check if the user doesn't care about the current update
-            if(!(ignmajor == major && ignminor == minor && ignpatch == patch && disablecheck))
+            if(!(ignmajor == major && ignminor == minor && ignpatch == patch))
             {
                 QMessageBox msgBox;
                 QPushButton *idontcarebutton = msgBox.addButton(tr("Don't show again"), QMessageBox::ActionRole);
@@ -1453,10 +1440,9 @@ void MainWindow::httpresponse(QNetworkReply *reply)
                 {
                     // save that the user don't want to get bothered about this update
                     settings.beginGroup("checkversion");
-                    settings.setValue("major", major);
-                    settings.setValue("minor", minor);
-                    settings.setValue("patch", minor);
-                    settings.setValue("disable", true);
+                    settings.setValue("ignmajor", major);
+                    settings.setValue("ignminor", minor);
+                    settings.setValue("ignpatch", patch);
                     settings.endGroup();
                 }
             }
@@ -1642,8 +1628,11 @@ void MainWindow::updatePlot(SqliteTableModel *model, bool update)
                 // set some graph styles, this could also be improved to let the user choose
                 // some styling
                 graph->setData(xdata, ydata);
-                graph->setLineStyle(QCPGraph::lsLine);
-                graph->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssDisc, 5));
+                graph->setLineStyle((QCPGraph::LineStyle) ui->comboLineType->currentIndex());
+                // WARN: ssDot is removed
+                int shapeIdx = ui->comboPointShape->currentIndex();
+                if (shapeIdx > 0) shapeIdx += 1;
+                graph->setScatterStyle(QCPScatterStyle((QCPScatterStyle::ScatterShape)shapeIdx, 5));
 
                 // gather Y label column names
                 yAxisLabels << model->headerData(y, Qt::Horizontal).toString();
@@ -1733,9 +1722,8 @@ void MainWindow::on_treePlotColumns_itemDoubleClicked(QTreeWidgetItem *item, int
 
 void MainWindow::on_butSavePlot_clicked()
 {
-    QString fileName = QFileDialog::getSaveFileName(this,
+    QString fileName = FileDialog::getSaveFileName(this,
                                                     tr("Choose a filename to save under"),
-                                                    PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                                                     tr("PNG(*.png);;JPG(*.jpg);;PDF(*.pdf);;BMP(*.bmp);;All Files(*)")
                                                     );
     if(!fileName.isEmpty())
@@ -1781,7 +1769,22 @@ void MainWindow::on_actionWebsite_triggered()
 
 void MainWindow::updateBrowseDataColumnWidth(int section, int /*old_size*/, int new_size)
 {
-    browseTableColumnWidths[ui->comboBrowseTable->currentText()][section] = new_size;
+    QSet<int> selectedCols(ui->dataTable->selectedCols());
+    QString tableName(ui->comboBrowseTable->currentText());
+    if (!selectedCols.contains(section))
+    {
+        browseTableColumnWidths[tableName][section] = new_size;
+    }
+    else
+    {
+        ui->dataTable->blockSignals(true);
+        foreach (int col, selectedCols)
+        {
+            ui->dataTable->setColumnWidth(col, new_size);
+            browseTableColumnWidths[tableName][col] = new_size;
+        }
+        ui->dataTable->blockSignals(false);
+    }
 }
 
 bool MainWindow::loadProject(QString filename)
@@ -1789,9 +1792,8 @@ bool MainWindow::loadProject(QString filename)
     // Show the open file dialog when no filename was passed as parameter
     if(filename.isEmpty())
     {
-        filename = QFileDialog::getOpenFileName(this,
+        filename = FileDialog::getOpenFileName(this,
                                                 tr("Choose a file to open"),
-                                                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                                                 tr("DB Browser for SQLite project file (*.sqbpro)"));
     }
 
@@ -1892,7 +1894,7 @@ bool MainWindow::loadProject(QString filename)
                             // SQL editor tab
                             unsigned int index = openSqlTab();
                             ui->tabSqlAreas->setTabText(index, xml.attributes().value("name").toString());
-                            qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->widget(index))->getEditor()->setPlainText(xml.readElementText());
+                            qobject_cast<SqlExecutionArea*>(ui->tabSqlAreas->widget(index))->getEditor()->setText(xml.readElementText());
                         } else if(xml.name() == "current_tab") {
                             // Currently selected tab
                             ui->tabSqlAreas->setCurrentIndex(xml.attributes().value("id").toString().toInt());
@@ -1929,9 +1931,8 @@ static void saveDbTreeState(const QTreeView* tree, QXmlStreamWriter& xml, QModel
 
 void MainWindow::saveProject()
 {
-    QString filename = QFileDialog::getSaveFileName(this,
+    QString filename = FileDialog::getSaveFileName(this,
                                                     tr("Choose a filename to save under"),
-                                                    PreferencesDialog::getSettingsValue("db", "defaultlocation").toString(),
                                                     tr("DB Browser for SQLite project file (*.sqbpro)")
                                                     );
     if(!filename.isEmpty())
@@ -2013,10 +2014,9 @@ void MainWindow::saveProject()
 void MainWindow::fileAttach()
 {
     // Get file name of database to attach
-    QString file = QFileDialog::getOpenFileName(
+    QString file = FileDialog::getOpenFileName(
                 this,
-                tr("Choose a database file"),
-                PreferencesDialog::getSettingsValue("db", "defaultlocation").toString());
+                tr("Choose a database file"));
     if(!QFile::exists(file))
         return;
 
@@ -2087,3 +2087,63 @@ void MainWindow::editEncryption()
     }
 #endif
 }
+
+void MainWindow::switchToBrowseDataTab()
+{
+    if(!ui->dbTreeWidget->selectionModel()->hasSelection())
+        return;
+
+    QString tableToBrowse = ui->dbTreeWidget->model()->data(ui->dbTreeWidget->currentIndex().sibling(ui->dbTreeWidget->currentIndex().row(), 0)).toString();
+
+    ui->mainTab->setCurrentIndex(1);
+    ui->comboBrowseTable->setCurrentIndex(ui->comboBrowseTable->findText(tableToBrowse));
+    populateTable(tableToBrowse);
+}
+
+void MainWindow::on_buttonClearFilters_clicked()
+{
+    ui->dataTable->filterHeader()->clearFilters();
+}
+
+void MainWindow::copyCurrentCreateStatement()
+{
+    // Cancel if no field is currently selected
+    if(!ui->dbTreeWidget->selectionModel()->hasSelection())
+        return;
+
+    // Get the CREATE statement from the Schema column
+    QString stmt = ui->dbTreeWidget->model()->data(ui->dbTreeWidget->currentIndex().sibling(ui->dbTreeWidget->currentIndex().row(), 3)).toString();
+
+    // Copy the statement to the global application clipboard
+    QApplication::clipboard()->setText(stmt);
+}
+
+void MainWindow::on_comboLineType_currentIndexChanged(int index)
+{
+    Q_ASSERT(index >= QCPGraph::lsNone &&
+             index <= QCPGraph::lsImpulse);
+    QCPGraph::LineStyle lineStyle = (QCPGraph::LineStyle) index;
+    for (int i = 0, ie = ui->plotWidget->graphCount(); i < ie; ++i)
+    {
+        QCPGraph * graph = ui->plotWidget->graph(i);
+        if (graph)
+            graph->setLineStyle(lineStyle);
+    }
+    ui->plotWidget->replot();
+}
+
+void MainWindow::on_comboPointShape_currentIndexChanged(int index)
+{
+    // WARN: because ssDot point shape is removed
+    if (index > 0) index += 1;
+    Q_ASSERT(index >= QCPScatterStyle::ssNone &&
+             index <  QCPScatterStyle::ssPixmap);
+    QCPScatterStyle::ScatterShape shape = (QCPScatterStyle::ScatterShape) index;
+    for (int i = 0, ie = ui->plotWidget->graphCount(); i < ie; ++i)
+    {
+        QCPGraph * graph = ui->plotWidget->graph(i);
+        if (graph)
+            graph->setScatterStyle(QCPScatterStyle(shape, 5));
+    }
+    ui->plotWidget->replot();
+}
