@@ -35,6 +35,26 @@ void collation_needed(void* /*pData*/, sqlite3* db, int eTextRep, const char* sC
         sqlite3_create_collation(db, sCollationName, eTextRep, NULL, collCompare);
 }
 
+
+static void regexp(sqlite3_context* ctx, int /*argc*/, sqlite3_value* argv[])
+{
+    // Get arguments and check their values
+    QRegExp arg1((const char*)sqlite3_value_text(argv[0]));
+    QString arg2((const char*)sqlite3_value_text(argv[1]));
+    if(!arg1.isValid())
+        return sqlite3_result_error(ctx, "invalid operand", -1);
+
+    // Set the pattern matching syntax to a Perl-like one. This is the default in Qt 4.x but Qt 5
+    // changes this to a greedy one (QRegExp::RegExp2). To make sure the behaviour of our application
+    // doesn't change depending on the build environment, we make sure to always set the same pattern
+    // matching syntax.
+    arg1.setPatternSyntax(QRegExp::RegExp);
+
+    // Perform the actual matching and return the result. Note that Qt's QRegExp returns -1 if the regex
+    // doesn't match and the position in the string otherwise; SQLite expects a 0 for not found and a 1 for found.
+    sqlite3_result_int(ctx, arg1.indexIn(arg2) >= 0);
+}
+
 bool DBBrowserDB::isOpen ( ) const
 {
     return _db!=0;
@@ -74,7 +94,7 @@ bool DBBrowserDB::open(const QString& db)
     {
         sqlite3_key(_db, cipher->password().toUtf8(), cipher->password().toUtf8().length());
         if(cipher->pageSize() != 1024)
-            sqlite3_exec(_db, QString("PRAGMA cipher_page_size = %1;").arg(cipher->pageSize()).toUtf8(), NULL, NULL, NULL);
+            executeSQL(QString("PRAGMA cipher_page_size = %1;").arg(cipher->pageSize()), false, false);
     }
 #endif
     delete cipher;
@@ -93,6 +113,10 @@ bool DBBrowserDB::open(const QString& db)
         // Enable extension loading
         sqlite3_enable_load_extension(_db, 1);
 
+        // Register REGEXP function
+        if(settings.value("/extensions/disableregex", false) == false)
+            sqlite3_create_function(_db, "REGEXP", 2, SQLITE_UTF8, NULL, regexp, NULL, NULL);
+
         curDBFilename = db;
         return true;
     } else {
@@ -212,13 +236,11 @@ bool DBBrowserDB::setRestorePoint(const QString& pointname)
     if(savepointList.contains(pointname))
         return true;
 
-    if(_db)
-    {
-        QString query = QString("SAVEPOINT %1;").arg(pointname);
-        sqlite3_exec(_db, query.toUtf8(), NULL, NULL, NULL);
-        savepointList.append(pointname);
-        emit dbChanged(getDirty());
-    }
+    QString query = QString("SAVEPOINT %1;").arg(pointname);
+    executeSQL(query, false, false);
+    savepointList.append(pointname);
+    emit dbChanged(getDirty());
+
     return true;
 }
 
@@ -227,13 +249,11 @@ bool DBBrowserDB::save(const QString& pointname)
     if(!isOpen() || savepointList.contains(pointname) == false)
         return false;
 
-    if(_db)
-    {
-        QString query = QString("RELEASE %1;").arg(pointname);
-        sqlite3_exec(_db, query.toUtf8(), NULL,NULL,NULL);
-        savepointList.removeAll(pointname);
-        emit dbChanged(getDirty());
-    }
+    QString query = QString("RELEASE %1;").arg(pointname);
+    executeSQL(query, false, false);
+    savepointList.removeAll(pointname);
+    emit dbChanged(getDirty());
+
     return true;
 }
 
@@ -242,15 +262,13 @@ bool DBBrowserDB::revert(const QString& pointname)
     if(!isOpen() || savepointList.contains(pointname) == false)
         return false;
 
-    if(_db)
-    {
-        QString query = QString("ROLLBACK TO SAVEPOINT %1;").arg(pointname);
-        sqlite3_exec(_db, query.toUtf8(), NULL, NULL, NULL);
-        query = QString("RELEASE %1;").arg(pointname);
-        sqlite3_exec(_db, query.toUtf8(), NULL, NULL, NULL);
-        savepointList.removeAll(pointname);
-        emit dbChanged(getDirty());
-    }
+    QString query = QString("ROLLBACK TO SAVEPOINT %1;").arg(pointname);
+    executeSQL(query, false, false);
+    query = QString("RELEASE %1;").arg(pointname);
+    executeSQL(query, false, false);
+    savepointList.removeAll(pointname);
+    emit dbChanged(getDirty());
+
     return true;
 }
 
@@ -309,9 +327,9 @@ bool DBBrowserDB::create ( const QString & db)
         // force sqlite3 do write proper file header
         // if we don't create and drop the table we might end up
         // with a 0 byte file, if the user cancels the create table dialog
-        sqlite3_exec(_db, "CREATE TABLE notempty (id integer primary key);", NULL, NULL, NULL);
-        sqlite3_exec(_db, "DROP TABLE notempty", NULL, NULL, NULL);
-        sqlite3_exec(_db, "COMMIT;", NULL, NULL, NULL);
+        executeSQL("CREATE TABLE notempty (id integer primary key);", false, false);
+        executeSQL("DROP TABLE notempty;", false, false);
+        executeSQL("COMMIT;", false, false);
 
         ok = true;
         curDBFilename = db;
@@ -354,7 +372,11 @@ bool DBBrowserDB::close()
     return true;
 }
 
-bool DBBrowserDB::dump(const QString& filename)
+bool DBBrowserDB::dump(const QString& filename,
+    const QStringList & tablesToDump,
+    bool insertColNames,
+    bool insertNewSyntx,
+    bool exportSchemaOnly)
 {
     // Open file
     QFile file(filename);
@@ -396,12 +418,22 @@ bool DBBrowserDB::dump(const QString& filename)
         // Loop through all tables first as they are required to generate views, indices etc. later
         for(QList<DBBrowserObject>::ConstIterator it=tables.begin();it!=tables.end();++it)
         {
+            if (tablesToDump.indexOf(it->getTableName()) == -1)
+                continue;
+
             // Write the SQL string used to create this table to the output file
             stream << it->getsql() << ";\n";
 
+            if (exportSchemaOnly)
+                continue;
+
+            // get columns
+            QStringList cols(it->table.fieldNames());
+
             QString sQuery = QString("SELECT * FROM `%1`;").arg(it->getTableName());
             QByteArray utf8Query = sQuery.toUtf8();
             sqlite3_stmt *stmt;
+            QString lineSep(QString(")%1\n").arg(insertNewSyntx?',':';'));
 
             int status = sqlite3_prepare_v2(this->_db, utf8Query.data(), utf8Query.size(), &stmt, NULL);
             if(SQLITE_OK == status)
@@ -411,7 +443,20 @@ bool DBBrowserDB::dump(const QString& filename)
                 qApp->processEvents();
                 while(sqlite3_step(stmt) == SQLITE_ROW)
                 {
-                    stream << "INSERT INTO `" << it->getTableName() << "` VALUES (";
+                    if (counter) stream << lineSep;
+
+                    if (!insertNewSyntx || !counter)
+                    {
+                        stream << "INSERT INTO `" << it->getTableName() << '`';
+                        if (insertColNames)
+                            stream << " (" << cols.join(",") << ")";
+                        stream << " VALUES (";
+                    }
+                    else
+                    {
+                        stream << " (";
+                    }
+
                     for (int i = 0; i < columns; ++i)
                     {
                         int fieldsize = sqlite3_column_bytes(stmt, i);
@@ -435,6 +480,12 @@ bool DBBrowserDB::dump(const QString& filename)
                             case SQLITE_NULL:
                                 stream << "NULL";
                             break;
+                            case SQLITE_FLOAT:
+                                if(bcontent.indexOf("Inf") != -1)
+                                    stream << "'" << bcontent << "'";
+                                else
+                                    stream << bcontent;
+                            break;
                             default:
                                 stream << bcontent;
                             }
@@ -442,7 +493,6 @@ bool DBBrowserDB::dump(const QString& filename)
                         if(i != columns - 1)
                             stream << ',';
                     }
-                    stream << ");\n";
 
                     progress.setValue(++numRecordsCurrent);
                     if(counter % 5000 == 0)
@@ -458,6 +508,7 @@ bool DBBrowserDB::dump(const QString& filename)
                         return false;
                     }
                 }
+                if (counter > 0) stream << ");\n";
             }
             sqlite3_finalize(stmt);
         }
@@ -492,23 +543,23 @@ bool DBBrowserDB::executeSQL ( const QString & statement, bool dirtyDB, bool log
     char *errmsg;
     bool ok = false;
 
-    if (!isOpen()) return false;
+    if (!isOpen())
+        return false;
 
-    if (_db){
-        if (logsql) logSQL(statement, kLogMsg_App);
-        if (dirtyDB) setRestorePoint();
-        if (SQLITE_OK==sqlite3_exec(_db,statement.toUtf8(),
-                                    NULL,NULL,&errmsg)){
-            ok=true;
-        }
-    }
+    if (logsql) logSQL(statement, kLogMsg_App);
+    if (dirtyDB) setRestorePoint();
+    if (SQLITE_OK == sqlite3_exec(_db, statement.toUtf8(), NULL, NULL, &errmsg))
+        ok = true;
 
-    if (!ok){
-        lastErrorMessage = QString::fromUtf8(errmsg);
-        qWarning() << "executeSQL: " << statement << "->" << lastErrorMessage;
+    if(ok)
+    {
+        lastErrorMessage = tr("no error");
+        return true;
+    } else {
+        lastErrorMessage = QString("%1 (%2)").arg(QString::fromUtf8(errmsg)).arg(statement);
+        qWarning() << "executeSQL: " << statement << "->" << errmsg;
         return false;
     }
-    return true;
 }
 
 bool DBBrowserDB::executeMultiSQL(const QString& statement, bool dirty, bool log)
@@ -578,9 +629,9 @@ bool DBBrowserDB::executeMultiSQL(const QString& statement, bool dirty, bool log
     return true;
 }
 
-bool DBBrowserDB::getRow(const QString& sTableName, int64_t rowid, QList<QByteArray>& rowdata)
+bool DBBrowserDB::getRow(const QString& sTableName, const QString& rowid, QList<QByteArray>& rowdata)
 {
-    QString sQuery = QString("SELECT * FROM `%1` WHERE `%2`=%3;").arg(sTableName).arg(getObjectByName(sTableName).table.rowidColumn()).arg(rowid);
+    QString sQuery = QString("SELECT * FROM `%1` WHERE `%2`='%3';").arg(sTableName).arg(getObjectByName(sTableName).table.rowidColumn()).arg(rowid);
     QByteArray utf8Query = sQuery.toUtf8();
     sqlite3_stmt *stmt;
     bool ret = false;
@@ -592,7 +643,18 @@ bool DBBrowserDB::getRow(const QString& sTableName, int64_t rowid, QList<QByteAr
         while(sqlite3_step(stmt) == SQLITE_ROW)
         {
             for (int i = 0; i < sqlite3_column_count(stmt); ++i)
-                rowdata.append(QByteArray(static_cast<const char*>(sqlite3_column_blob(stmt, i)), sqlite3_column_bytes(stmt, i)));
+            {
+                if(sqlite3_column_type(stmt, i) == SQLITE_NULL)
+                {
+                    rowdata.append(QByteArray());
+                } else {
+                    int bytes = sqlite3_column_bytes(stmt, i);
+                    if(bytes)
+                        rowdata.append(QByteArray(static_cast<const char*>(sqlite3_column_blob(stmt, i)), bytes));
+                    else
+                        rowdata.append(QByteArray(""));
+                }
+            }
             ret = true;
         }
     }
@@ -601,12 +663,12 @@ bool DBBrowserDB::getRow(const QString& sTableName, int64_t rowid, QList<QByteAr
     return ret;
 }
 
-int64_t DBBrowserDB::max(const sqlb::Table& t, sqlb::FieldPtr field) const
+QString DBBrowserDB::max(const sqlb::Table& t, sqlb::FieldPtr field) const
 {
-    QString sQuery = QString("SELECT MAX(`%2`) from `%1`;").arg(t.name()).arg(field->name());
+    QString sQuery = QString("SELECT MAX(CAST(`%2` AS INTEGER)) FROM `%1`;").arg(t.name()).arg(field->name());
     QByteArray utf8Query = sQuery.toUtf8();
     sqlite3_stmt *stmt;
-    int64_t ret = 0;
+    QString ret = "0";
 
     int status = sqlite3_prepare_v2(_db, utf8Query, utf8Query.size(), &stmt, NULL);
     if(SQLITE_OK == status)
@@ -616,7 +678,7 @@ int64_t DBBrowserDB::max(const sqlb::Table& t, sqlb::FieldPtr field) const
         {
             if(sqlite3_column_count(stmt) == 1)
             {
-                ret = sqlite3_column_int64(stmt, 0);
+                ret = QString::fromUtf8(reinterpret_cast<const char*>(sqlite3_column_text(stmt, 0)));
             }
         }
     }
@@ -625,7 +687,7 @@ int64_t DBBrowserDB::max(const sqlb::Table& t, sqlb::FieldPtr field) const
     return ret;
 }
 
-QString DBBrowserDB::emptyInsertStmt(const sqlb::Table& t, int64_t pk_value) const
+QString DBBrowserDB::emptyInsertStmt(const sqlb::Table& t, const QString& pk_value) const
 {
     QString stmt = QString("INSERT INTO `%1`").arg(t.name());
 
@@ -633,31 +695,22 @@ QString DBBrowserDB::emptyInsertStmt(const sqlb::Table& t, int64_t pk_value) con
     QStringList fields;
     foreach(sqlb::FieldPtr f, t.fields()) {
         if(f->primaryKey()) {
-            if(f->isInteger())
-            {
-                fields << f->name();
+            fields << f->name();
 
-                if(pk_value != -1)
-                    vals << QString::number(pk_value);
+            if(!pk_value.isNull())
+            {
+                vals << pk_value;
+            } else {
+                if(f->notnull())
+                {
+                    QString maxval = this->max(t, f);
+                    vals << QString::number(maxval.toLongLong() + 1);
+                }
                 else
                 {
-                    if(f->notnull())
-                    {
-                        int64_t maxval = this->max(t, f);
-                        vals << QString::number(maxval + 1);
-                    }
-                    else
-                    {
-                        vals << "NULL";
-                    }
+                    vals << "NULL";
                 }
             }
-            else
-            {
-                fields << f->name();
-
-                vals << "''";
-            }
         } else if(f->notnull() && f->defaultValue().length() == 0) {
             fields << f->name();
 
@@ -689,84 +742,82 @@ QString DBBrowserDB::emptyInsertStmt(const sqlb::Table& t, int64_t pk_value) con
     return stmt;
 }
 
-int64_t DBBrowserDB::addRecord(const QString& sTableName)
+QString DBBrowserDB::addRecord(const QString& sTableName)
 {
-    char *errmsg;
-    if (!isOpen()) return false;
+    if (!isOpen()) return QString();
 
     sqlb::Table table = getObjectByName(sTableName).table;
 
     // For tables without rowid we have to set the primary key by ourselves. We do so by querying for the largest value in the PK column
     // and adding one to it.
     QString sInsertstmt;
-    int64_t pk_value;
+    QString pk_value;
     if(table.isWithoutRowidTable())
     {
-        SqliteTableModel m(this, this);
-        m.setQuery(QString("SELECT MAX(`%1`) FROM `%2`;").arg(table.rowidColumn()).arg(sTableName));
-        pk_value = m.data(m.index(0, 0)).toLongLong() + 1;
+        pk_value = QString::number(max(table, table.fields().at(table.findField(table.rowidColumn()))).toLongLong() + 1);
         sInsertstmt = emptyInsertStmt(table, pk_value);
     } else {
         sInsertstmt = emptyInsertStmt(table);
     }
 
-    lastErrorMessage = "";
-    logSQL(sInsertstmt, kLogMsg_App);
-    setRestorePoint();
-
-    if (SQLITE_OK != sqlite3_exec(_db, sInsertstmt.toUtf8(), NULL, NULL, &errmsg))
+    if(!executeSQL(sInsertstmt))
     {
-        lastErrorMessage = QString::fromUtf8(errmsg);
         qWarning() << "addRecord: " << lastErrorMessage;
-        return -1;
+        return QString();
     } else {
         if(table.isWithoutRowidTable())
             return pk_value;
         else
-            return sqlite3_last_insert_rowid(_db);
+            return QString::number(sqlite3_last_insert_rowid(_db));
     }
 }
 
-bool DBBrowserDB::deleteRecord(const QString& table, int64_t rowid)
+bool DBBrowserDB::deleteRecord(const QString& table, const QString& rowid)
 {
-    char * errmsg;
     if (!isOpen()) return false;
     bool ok = false;
     lastErrorMessage = QString("no error");
 
-    QString statement = QString("DELETE FROM `%1` WHERE `%2`=%3;").arg(table).arg(getObjectByName(table).table.rowidColumn()).arg(rowid);
+    QString statement = QString("DELETE FROM `%1` WHERE `%2`='%3';").arg(table).arg(getObjectByName(table).table.rowidColumn()).arg(rowid);
+    if(executeSQL(statement))
+        ok = true;
+    else
+        qWarning() << "deleteRecord: " << lastErrorMessage;
 
-    if (_db){
-        logSQL(statement, kLogMsg_App);
-        setRestorePoint();
-        if (SQLITE_OK==sqlite3_exec(_db,statement.toUtf8(),
-                                    NULL,NULL,&errmsg)){
-            ok=true;
-        } else {
-            lastErrorMessage = QString::fromUtf8(errmsg);
-            qWarning() << "deleteRecord: " << lastErrorMessage;
-        }
-    }
     return ok;
 }
 
-bool DBBrowserDB::updateRecord(const QString& table, const QString& column, int64_t row, const QByteArray& value)
+bool DBBrowserDB::updateRecord(const QString& table, const QString& column, const QString& rowid, const QByteArray& value, bool itsBlob)
 {
     if (!isOpen()) return false;
 
     lastErrorMessage = QString("no error");
 
-    QString sql = QString("UPDATE `%1` SET `%2`=? WHERE `%3`=%4;").arg(table).arg(column).arg(getObjectByName(table).table.rowidColumn()).arg(row);
+    QString sql = QString("UPDATE `%1` SET `%2`=? WHERE `%3`='%4';").arg(table).arg(column).arg(getObjectByName(table).table.rowidColumn()).arg(rowid);
 
     logSQL(sql, kLogMsg_App);
     setRestorePoint();
 
+    // If we get a NULL QByteArray we insert a NULL value, and for that
+    // we can pass NULL to sqlite3_bind_text() so that it behaves like sqlite3_bind_null()
+    const char *rawValue = value.isNull() ? NULL : value.constData();
+
     sqlite3_stmt* stmt;
     int success = 1;
     if(sqlite3_prepare_v2(_db, sql.toUtf8(), -1, &stmt, 0) != SQLITE_OK)
         success = 0;
-    if(success == 1 && sqlite3_bind_text(stmt, 1, value.constData(), value.length(), SQLITE_STATIC) != SQLITE_OK)
-        success = -1;
+    if(success == 1) {
+        if(itsBlob)
+        {
+            if(sqlite3_bind_blob(stmt, 1, rawValue, value.length(), SQLITE_STATIC))
+                success = -1;
+        }
+        else
+        {
+            if(sqlite3_bind_text(stmt, 1, rawValue, value.length(), SQLITE_STATIC))
+                success = -1;
+        }
+    }
     if(success == 1 && sqlite3_step(stmt) != SQLITE_DONE)
         success = -1;
     if(success != 0 && sqlite3_finalize(stmt) != SQLITE_OK)
