@@ -3,6 +3,7 @@
 
 #include <QAbstractTableModel>
 #include <QStringList>
+#include <QVector>
 
 class DBBrowserDB;
 
@@ -50,19 +51,21 @@ private:
     void clearCache();
 
     void buildQuery();
-    QStringList getColumns(const QString sQuery);
+    QStringList getColumns(const QString& sQuery, QVector<int>& fieldsTypes);
     int getQueryRowCount();
 
     DBBrowserDB* m_db;
     int m_rowCount;
     QStringList m_headers;
-    QList<QByteArrayList> m_data;
+    typedef QList<QByteArrayList> DataType;
+    DataType m_data;
 
     QString m_sQuery;
     QString m_sTable;
     int m_iSortColumn;
     QString m_sSortOrder;
     QMap<int, QString> m_mWhere;
+    QVector<int> m_vDataTypes;
 
     /**
      * @brief m_chunkSize Size of the next chunk fetch more will try to fetch.
